# type: ignore
# ^ that's necessary to prevent a false linting error of some kind
import asyncio
import urllib.parse
from time import perf_counter

import typer
from mcsniperpy.util.logs_manager import Color as color
from mcsniperpy.util.logs_manager import Logger as log


async def check(url: str, iterations: int):
    async def ping():
        try:
            uri = urllib.parse.urlparse(url)
            reader, writer = await asyncio.open_connection(uri.hostname, 443, ssl=False)
            writer.write(
                f"GET {uri.path or '/'} HTTP/1.1\r\nHost:{uri.hostname}\r\n\r\n".encode()
            )

            start = perf_counter()
            await writer.drain()

            _ = await reader.read(100)
            end = perf_counter()
            return round((end - start) * 1000)
        # pylint: disable=invalid-name, broad-except
        except Exception as e:
            log.error("Failed to connect to URL. error code: " + str(e))

    pings = []

    with typer.progressbar(
        range(iterations),
        fill_char="█",
        empty_char=" ",
        color=10,
        show_eta=False,
        bar_template="%(label)s  %(bar)s  %(info)s",
    ) as progress:
        for _ in progress:
            pings.append(await ping())
            await asyncio.sleep(0.01)

    print()
    log.info(f"Host {color.l_cyan}» {color.blue}{urllib.parse.urlparse(url).hostname}")
    log.info(f"Ping {color.l_cyan}» {color.blue}{sum(pings) / 5}ms")


async def ping_test(iterations):
    print()
    await check("https://api.minecraftservices.com/minecraft", iterations)
    # await check("https://api.mojang.com") ### Literally useless lol



import re
from datetime import datetime

from mcsniperpy.util.logs_manager import Color as color
from mcsniperpy.util.logs_manager import Logger as log
from mcsniperpy.util.request_manager import RequestManager


async def announce(
    username: str,
    authorization: str,
    session: RequestManager,
    full_url: str = "https://announcements-api.herokuapp.com/api/v1/announce",
) -> bool:
    headers = {"Authorization": authorization}
    json = {"name": username}
    resp, _, resp_json = await session.post(full_url, json=json, headers=headers)
    if resp.status < 300:
        log.info(f"{color.white}[{color.l_green}success{color.white}]{color.reset}")
        return True

    log.error(f"Failed to announce snipe | {resp.status} | {resp_json}")
    return False


DISCORD_RESP_CODES = {
    200: "ok",
    201: "created",
    204: "ok (no content)",
    304: "not modified",
    400: "bad request",
    401: "unauthorized",
    403: "forbidden",
    404: "not found",
    405: "method not allowed",
    429: "too many requests",
    502: "gateway unavailable",
    500: "server error",
}


async def gen_webhook_desc(desc_format, name, session: RequestManager):
    resp, _, resp_json = await session.get(
        f"https://api.kqzz.me/api/namemc/searches/{name}"
    )

    searches: int = 0

    if resp.status < 300:
        searches = resp_json["searches"]
    else:
        searches = 0

    final_format = desc_format.format(
        name=name, searches=searches, namemcLink=f"https://namemc.com/profile/{name}"
    )

    return final_format


async def webhook_announce(
    webhook_url: str,
    session: RequestManager,
    embed_description: str,
    embed_title: str,
    prename: bool = False,
    embed_color: int = 0x779ABD,
):
    regex = re.compile(
        r"http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|"
        r"[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+"
    )
    if regex.match(webhook_url):
        action = "prenamed" if prename else "sniped"
        embed_content ={
            "content": None,
            "embeds": [                {
                    "title": embed_title,
                    "description": embed_description,
                    "color": embed_color,
                    "footer": {
                        "text": f"{action.title()} with MCsniperPY",
                        "icon_url": "https://mcsniperpy.com/img/old-logo.png",
                    },
                    "timestamp": datetime.utcnow().strftime(
                        "%Y-%m-%dT%H:%M:%S.000Z"
                    ),
                }
            ],
        }
        resp, _, _ = await session.post(webhook_url, json=embed_content)
        if resp.status < 300:
            log.info(
                f"{color.white}[{color.l_green}success{color.white}]{color.reset} "
                "successfully sent discord webhook!"
            )
            return True

        log.error(
            f"failed to send Discord webhook | {resp.status} | "
            f"{DISCORD_RESP_CODES.get(resp.status, 'unknown')}"
        )



import asyncio
from datetime import datetime

import aiohttp
from bs4 import BeautifulSoup

from mcsniperpy.util.logs_manager import Logger as log, Color as color
from mcsniperpy.util.request_manager import RequestManager
from mcsniperpy.util.utils import close


async def namemc_timing(username: str, session: RequestManager) -> float:
    resp, text, _ = await session.get(f"https://namemc.com/search?q={username}")

    if resp.status != 502:
        try:
            soup = BeautifulSoup(text, "html.parser")

            status_bar = soup.find(id="status-bar")
            info = status_bar.find_all("div", class_="col-sm-6 my-1")

            name_status = info[0].text.split("\n")[2].strip("*").lower()
            pretty_username = f"{color.l_cyan}{username}{color.reset}"
            pretty_status = f"{color.yellow if name_status == 'Unavailable' else color.green}{name_status}{color.reset}"

            if name_status == "unavailable":
                log.error(
                    f"failed to parse droptime for {pretty_username} through namemc"
                )
                log.info(
                    f"the username {pretty_username} is {pretty_status} at the moment which is most likely your problem!"
                )
                close(1)
            elif name_status == "available":
                log.info(
                    f"failed to parse droptime for {pretty_username} through namemc"
                )
                log.info(
                    f"the username {pretty_username} is {pretty_status} at the moment!"
                )
                log.info("go claim it without the sniper if possible.")
                close(1)
            elif name_status == "available later":
                snipe_time = soup.find("time", {"id": "availability-time"}).attrs[
                    "datetime"
                ]
                snipe_time = datetime.strptime(snipe_time, "%Y-%m-%dT%H:%M:%S.000Z")
                log.info(f"sniping {pretty_username} @ {snipe_time}")
                return snipe_time.timestamp()
            else:
                log.error(
                    "well this is awkward. You shouldn't be able to get here, but you are."
                )
                log.info(f"the status is {pretty_status}")
                close(1)
        # pylint: disable=broad-except
        except Exception as ex:
            # TODO: Need to find all the cases why this is triggered
            log.error(ex)
            log.error(
                "Couldn't get droptime from namemc. "
                "Maybe the name isn't dropping?"
            )
            close(0)
    else:
        log.error(f"failed to connect to {color.l_blue}namemc.com")
        close(1)

    close(1)
    return 0


async def teun_timing(
    username: str,
    session: RequestManager
) -> int:  # Returns a unix timestamp
    resp, _, resp_json = await session.get(
        f"https://mojang-api.teun.lol/droptime/{username}"
    )
    if resp.status < 300:
        log.info(
            f"sniping {color.cyan}{username}{color.reset} @ {datetime.fromtimestamp(resp_json['UNIX'])}"
        )
        return resp_json["UNIX"]

    log.error(
        f"failed to parse droptime for {color.l_cyan}{username}{color.reset} through teuns's MC API"
    )
    log.error(f"{resp_json['error']} | {resp.status}")

    close(0)
    return 0


async def coolkidmacho_timing(
        username: str,
        session: RequestManager
):
    resp, _, resp_json = await session.get(
        f"http://api.coolkidmacho.com/droptime/{username}"
    )

    if resp.status < 300:
        log.info(
            f"sniping {color.cyan}{username}{color.reset} @ {datetime.fromtimestamp(resp_json['UNIX'])}"
        )
        return resp_json["UNIX"]

    log.error(
        f"failed to parse droptime for {color.l_cyan}{username}{color.reset} through Coolkidmacho's droptime API"
    )
    log.error(f"{resp_json['error']} | {resp.status}")

    close(0)
    return 0


async def api_timing(
    username: str,
    session: RequestManager
) -> int:  # Returns a unix timestamp
    resp, _, resp_json = await session.get(
        f"https://api.kqzz.me/api/namemc/droptime/{username}"
    )
    # pylint: disable=no-else-return
    if resp_json.get("error", None) is None:
        log.info(
            f"sniping {color.cyan}{username}{color.reset} @ {datetime.fromtimestamp(resp_json['droptime'])}"
        )
        return resp_json["droptime"]
    else:
        log.error(
            f"failed to parse droptime for {color.l_cyan}{username}{color.reset} through Kqzz's MC API"
        )
        log.error(f"{resp_json['error']} | {resp.status}")
        close(0)

    close(0)
    return 0


def next_name(
    base_url: str = "https://api.kqzz.me",
    searches: int = 0,
    loop=asyncio.get_event_loop(),
):
    async def do_request():
        async with aiohttp.ClientSession() as session:
            resp = await session.get(
                base_url + "/api/namemc/upcoming?searches=" + str(searches)
            )
            upcoming_names = await resp.json()
        if len(upcoming_names) > 0:
            return upcoming_names[0]["name"]
        return None

    return loop.run_until_complete(do_request())



import asyncio
import math
import time
from statistics import mean

import aiohttp
from mcsniperpy.util import request_manager
from mcsniperpy.util import utils as util
from mcsniperpy.util.logs_manager import Color as color
from mcsniperpy.util.logs_manager import Logger as log


class OffsetCalculator:
    def __init__(self, req_count=2, accuracy=20, aim_for=0.15, do_log=True):
        self.session = request_manager.RequestManager(
            None
            # aiohttp.ClientSession(connector=aiohttp.TCPConnector(limit=300),headers={})
        )

        self.aim_for = aim_for

        self.accounts = util.parse_accs_string("email:pass")

        self.req_count = req_count

        self.log = log
        self.color = color

        self.offset = -100
        self.accuracy = accuracy

        self.do_log = do_log

    async def run(self):

        self.session.session = aiohttp.ClientSession(headers={})

        while True:
            droptime = math.floor(time.time() + 3)
            self.accounts = util.parse_accs_string("email:pass")
            if self.do_log:
                log.info(
                    f"testing offset {self.offset} in "
                    f"{round(droptime - time.time())} seconds"
                )
            res = await self.offset_test(droptime, "test", self.offset)
            if res is not True:
                self.offset += res
            else:
                return self.offset

    async def offset_test(self, droptime, target, offset):

        pre_snipe_coroutines = [
            acc.snipe_connect() for _ in range(self.req_count)
            for acc in self.accounts
        ]  # For later use

        now = time.time()
        time_until_connect = 0 if now > (droptime - 20) else (droptime - 20) - now

        await asyncio.sleep(time_until_connect)

        await asyncio.gather(*pre_snipe_coroutines)  # Connects

        snipe_coroutines = [
            acc.snipe(acc.readers_writers[i][1], do_log=False)
            for i in range(self.req_count)
            for acc in self.accounts
        ]  # Send requests

        while time.time() < droptime - offset / 1000:
            await asyncio.sleep(0.00001)  # bad timing solution but
            # it's fairly accurate

        # According to my tests this was .0004 seconds late while
        # the other method (asyncio.sleep) was .004 seconds late

        start = time.perf_counter()
        await asyncio.gather(*snipe_coroutines)  # Sends the snipe requests
        end = time.perf_counter()

        responses = await asyncio.gather(
            *[
                acc.snipe_read(
                    target,
                    acc.readers_writers[i][0],
                    acc.readers_writers[i][1],
                    do_log=self.do_log,
                )
                for i in range(self.req_count)
                for acc in self.accounts
            ]
        )  # Reads the responses
        # I swear I didn't try to make it ugly

        print(self.req_count / (end - start))

        target_time = droptime + self.aim_for  # Find the actual target time

        average_time = mean(
            [req_time for _, _, req_time in responses]
        )  # Finds the mean request time

        diff = -(target_time - average_time) * 1000
        if self.accuracy > diff > 0:
            if self.do_log:
                log.info(
                    f"{color.white}[{color.green}success{color.white}] "
                    f"{color.reset}{offset} is a good offset!"
                )
            return True

        return round(diff)

    def on_shutdown(self):
        if self.session.session is not None:
            asyncio.run(self.session.session.close())
        if self.log is not None:
            self.log.shutdown()



import colorama
from colorama import init

from typing import List

init(autoreset=True)
logs = List[str]


class Color:
    red = colorama.Fore.RED
    l_red = colorama.Fore.LIGHTRED_EX
    green = colorama.Fore.GREEN
    l_green = colorama.Fore.LIGHTGREEN_EX
    yellow = colorama.Fore.YELLOW
    l_yellow = colorama.Fore.LIGHTYELLOW_EX
    blue = colorama.Fore.BLUE
    l_blue = colorama.Fore.LIGHTBLUE_EX
    magenta = colorama.Fore.MAGENTA
    l_magenta = colorama.Fore.LIGHTMAGENTA_EX
    cyan = colorama.Fore.CYAN
    l_cyan = colorama.Fore.LIGHTCYAN_EX
    white = colorama.Fore.WHITE
    reset = colorama.Fore.RESET

    @classmethod
    def disable(cls):
        cls.red = ""
        cls.l_red = ""
        cls.green = ""
        cls.l_green = ""
        cls.yellow = ""
        cls.l_yellow = ""
        cls.blue = ""
        cls.l_blue = ""
        cls.magenta = ""
        cls.l_magenta = ""
        cls.cyan = ""
        cls.l_cyan = ""
        cls.white = ""
        cls.reset = ""

    @classmethod
    def enable(cls):
        cls.red = colorama.Fore.RED
        cls.l_red = colorama.Fore.LIGHTRED_EX
        cls.green = colorama.Fore.GREEN
        cls.l_green = colorama.Fore.LIGHTGREEN_EX
        cls.yellow = colorama.Fore.YELLOW
        cls.l_yellow = colorama.Fore.LIGHTYELLOW_EX
        cls.blue = colorama.Fore.BLUE
        cls.l_blue = colorama.Fore.LIGHTBLUE_EX
        cls.magenta = colorama.Fore.MAGENTA
        cls.l_magenta = colorama.Fore.LIGHTMAGENTA_EX
        cls.cyan = colorama.Fore.CYAN
        cls.l_cyan = colorama.Fore.LIGHTCYAN_EX
        cls.white = colorama.Fore.WHITE
        cls.reset = colorama.Fore.RESET


class Logger:
    logs = []

    info_enabled = True
    warn_enabled = True
    error_enabled = True
    success_enabled = True
    debug_enabled = False

    @classmethod
    def info(cls, message):

        formatted = f"{Color.white}[{Color.cyan}info{Color.white}] {message}"

        if cls.info_enabled:
            print(formatted)

    @classmethod
    def warn(cls, message):

        formatted = f"{Color.white}[{Color.yellow}warn{Color.white}] {message}"

        if cls.warn_enabled:
            print(formatted)

        cls.logs.append(formatted)

    @classmethod
    def error(cls, message):

        formatted = f"{Color.white}[{Color.red}error{Color.white}] {message}"

        if cls.error_enabled:
            print(formatted)

        cls.logs.append(formatted)

    @classmethod
    def success(cls, message):

        formatted = f"{Color.white}[{Color.green}success{Color.white}] {message}"

        if cls.success_enabled:
            print(f"{Color.white}[{Color.green}success{Color.white}] {message}")

        cls.logs.append(formatted)

    @classmethod
    def debug(cls, message):

        formatted = f"{Color.white}[{Color.l_cyan}debug{Color.white}] {message}"

        if cls.debug_enabled:
            print(formatted)

        cls.logs.append(formatted)

    @classmethod
    def input(cls, message) -> str:

        formatted = f"{Color.white}[{Color.cyan}input{Color.white}] {message}"

        print(formatted, end=" ")
        i = input()

        cls.logs.append(formatted + " " + i)

        return i

    @classmethod
    def yes_or_no(cls, message):
        i = str()
        while True:
            i = cls.input(
                f"{Color.white}[{Color.green}yes{Color.white}/{Color.l_red}no{Color.reset}]{Color.reset} "
                f"{message}"
            ).lower()
            if i in ("yes", "y"):
                return True
            elif i in ("no", "n"):
                return False
            else:
                cls.error(f"{i} is not a valid answer. please answer yes or no.")

    @classmethod
    def shutdown(cls):
        # Does nothing.
        # Could be used to:
        # send all logs to a discord channel,
        # send logs to a file uploading service to check back on later,
        # etc...
        pass
